#========================================================================================
# package      Hotfix
# state        public
#----------------------------------------------------------------------------------------
# description  Auxiliary functions for hotfix installation
#========================================================================================
package Hotfix;
use strict;
use base qw (Exporter);

our @EXPORT_OK  = qw (
    GetHostName
    MyIP
);

use Sys::Hostname;
use Sys::HostIP;

use DE_EPAGES::Core::API::PerlTools qw( GetIPAddresses );
use DE_EPAGES::WebInterface::API::Config qw(WebInterfaceConfigSectionIsHost WebInterfaceConfig);

#========================================================================================
# function     GetHostName
# state        public
#----------------------------------------------------------------------------------------
# syntax       $HostName = GetHostName();
#----------------------------------------------------------------------------------------
# description  return either 'localhost' (if single server installation, i.e. if all IPs
#               in WebInterface.conf belong to myself) or hostname() (if distributed
#               installation)
#----------------------------------------------------------------------------------------
# return       $HostName | 'localhost' or hostname() | string
#========================================================================================
sub GetHostName () {
    return hostname() unless -f "$ENV{EPAGES_CONFIG}/WebInterface.conf";

    my $IniConfig = WebInterfaceConfig();
    my @Servers = grep { WebInterfaceConfigSectionIsHost($_) } @{$IniConfig->sectionNames};

    return MyIP(@Servers) ? 'localhost' : hostname();
}

#========================================================================================
# function     MyIP
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsMine = MyIP(@IPs);
#----------------------------------------------------------------------------------------
# description  return either 1 (if all IPs of @IPs are mine) or 0 (also if @IPs are empty)
#----------------------------------------------------------------------------------------
# input        @IPs      | list of IPv4 or IPv6 | arrray
# return       $IsMine   | 1 (all IPs of @IPs are mine) or 0 | boolean
#========================================================================================
sub MyIP {
    my (@IPs) = @_;

    return 0 unless scalar @IPs;

    my %TestIPs;
    $TestIPs{$_} = 1 foreach @IPs;

    my $FoundIPs = Sys::HostIP->ips;
    foreach (keys %TestIPs) {
        delete $TestIPs{$_} if $_ ~~ @$FoundIPs;
    }
    return 1 unless scalar keys %TestIPs;

    $FoundIPs = GetIPAddresses();
    foreach my $Ver (qw(4 6)) {
        if (defined $FoundIPs->{$Ver}) {
            foreach (keys %TestIPs) {
                delete $TestIPs{$_} if $_ ~~ @{$FoundIPs->{$Ver}};
            }
        }
    }
    return 1 unless scalar keys %TestIPs;

    $FoundIPs = [];
    if ($^O eq 'MSWin32') {
        # ipconfig
        # Windows-IP-Konfiguration
        # Ethernetadapter LAN-Verbindung:
        #         Verbindungsspezifisches DNS-Suffix: intern.epages.de
        #         IP-Adresse. . . . . . . . . . . . : 172.20.8.17
        #         Subnetzmaske. . . . . . . . . . . : 255.255.0.0
        #         IP-Adresse. . . . . . . . . . . . : 2001:4bd8:2:2:dc65:9f29:be34:e079
        #         IP-Adresse. . . . . . . . . . . . : 2001:4bd8:2:2:18ba:aad3:1076:590d
        #         IP-Adresse. . . . . . . . . . . . : 2001:4bd8:2:2:21a:a0ff:fede:ab94
        #         IP-Adresse. . . . . . . . . . . . : fe80::21a:a0ff:fede:ab94%4
        #         Standardgateway . . . . . . . . . : 172.20.20.1
        #                                             fe80::214:a9ff:fef6:899a%4
        #
        # ...
        my $IfConfig = `ipconfig 2>nul`;
        my $Started = 0;
        foreach (split(/\n/, $IfConfig)) {
            $Started = 1 if /Ethernet/i;
            next unless $Started;
            $Started = 2 if /IP/;
            next if $Started < 2;
            last unless /\S/;
            push(@$FoundIPs, $1) if /IP.+\s:\s+([0-9a-fA-F:.]+)\s*$/;
        }
    } else {
        # ip addr show
        # 1: lo: <LOOPBACK,UP,LOWER_UP> mtu 16436 qdisc noqueue state UNKNOWN
        #     link/loopback 00:00:00:00:00:00 brd 00:00:00:00:00:00
        #     inet 127.0.0.1/8 brd 127.255.255.255 scope host lo
        #     inet6 ::1/128 scope host
        # 2: eth1: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc state UP qlen 1000
        #     link/ether 00:50:56:08:12:99 brd ff:ff:ff:ff:ff:ff
        #     inet 172.20.50.164/16 brd 172.20.255.255 scope global eth1
        #     inet6 2001:4bd8:2:2:250:56ff:fe08:1299/64 scope global dynamic
        #     inet6 fe80::250:56ff:fe08:1299/64 scope link
        # ...
        my $IfConfig = `PATH=/sbin:/usr/sbin:\$PATH ip addr show 2>/dev/null`;
        foreach (split(/\n/, $IfConfig)) {
            push(@$FoundIPs, $1) if /^\s+inet6?\s+([0-9a-fA-F:.]+)/;
        }
    }
    foreach (keys %TestIPs) {
        delete $TestIPs{$_} if $_ ~~ @$FoundIPs;
    }
    return scalar keys %TestIPs ? 0 : 1;
}

1;
